USE etlUnitTest;

INSERT INTO user_geo_ref
(user_geo_ref_id, zipcode, city, state, country)
VALUES
  (1, 55555, "Young America", "Minnesota", "United States"),
  (2, 12345, "Schenectady", "New York", "United States"),
  (3, 56789, "Colton", "California", "United States"),
  (4, 44444, "Newton Falls","Ohio", "United States")
;